@@ -11,14 +11,61 @@
 #import "iCalTree.h"
 #import "SelectionManager.h"
 #import "AlarmManager.h"
+#import "Alarm.h"
 #import "defines.h"
 
-@interface AppIcon : NSView
+@interface AppIcon : NSView <ConfigListener>
 {
-  NSTimer *timer;
+  ConfigManager *_cm;
+  NSDictionary *_attrs;
+  NSTimer *_timer;
+  BOOL _showDate;
+  BOOL _showTime;
+  NSImage *_bell;
 }
 @end
 @implementation AppIcon
+- (void)setup
+{
+  _showDate = [[_cm objectForKey:APPICON_DATE] boolValue];
+  _showTime = [[_cm objectForKey:APPICON_TIME] boolValue];
+  if (_showTime) {
+    if (_timer == nil)
+      _timer = [NSTimer scheduledTimerWithTimeInterval:1
+						target:self
+					      selector:@selector(secondChanged:)
+					      userInfo:nil
+					       repeats:YES];
+  } else {
+    [_timer invalidate];
+    _timer = nil;
+  }
+  [self setNeedsDisplay:YES];
+}
+- (void)dealloc
+{
+  [_cm unregisterClient:self];
+  [_attrs release];
+  [_timer invalidate];
+  [super dealloc];
+}
+- (id)initWithFrame:(NSRect)frame
+{
+  self = [super initWithFrame:frame];
+  if (self) {
+    _attrs = [[NSDictionary alloc] initWithObjectsAndKeys:[NSFont systemFontOfSize:8],NSFontAttributeName,nil];
+    _bell = [NSImage imageNamed:@"bell"];
+    _cm = [ConfigManager globalConfig];
+    [_cm registerClient:self forKey:APPICON_DATE];
+    [_cm registerClient:self forKey:APPICON_TIME];
+    [self setup];
+  }
+  return self;
+}
+- (void)config:(ConfigManager *)config dataDidChangedForKey:(NSString *)key
+{
+  [self setup];
+}
 - (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
 {
   return YES;
@@ -29,35 +76,61 @@
 }
 - (void)drawRect:(NSRect)rect
 {
-  ConfigManager *config = [ConfigManager globalConfig];
+  NSUserDefaults *def = [NSUserDefaults standardUserDefaults];
   NSCalendarDate *now = [[Date now] calendarDate];
-  NSDictionary *attrs;
   NSString *aString;
 
-  attrs = [NSMutableDictionary dictionaryWithObject:[NSFont systemFontOfSize: 8]  forKey:NSFontAttributeName];
-  if ([[config objectForKey:APPICON_DATE] boolValue]) {
-    aString = [now descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSShortDateFormatString]];
-    [aString drawAtPoint:NSMakePoint(8, 3) withAttributes:attrs];
+  if (_showDate) {
+    aString = [now descriptionWithCalendarFormat:[def objectForKey:NSShortDateFormatString]];
+    [aString drawAtPoint:NSMakePoint(8, 3) withAttributes:_attrs];
   }
-  if ([[config objectForKey:APPICON_TIME] boolValue]) {
-    aString = [now descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSTimeFormatString]];
-    [aString drawAtPoint:NSMakePoint(11, 48) withAttributes:attrs];
-    if (timer == nil)
-      timer = [NSTimer scheduledTimerWithTimeInterval:1
-         			       	       target:self
-		                             selector:@selector(secondChanged:)
-				             userInfo:nil
-				              repeats:YES];
-  } else {
-    if (timer)
-      [timer invalidate];
-    timer = nil;
+  if (_showTime) {
+    aString = [now descriptionWithCalendarFormat:[def objectForKey:NSTimeFormatString]];
+    [aString drawAtPoint:NSMakePoint(11, 49) withAttributes:_attrs];
   }
+  //[_bell dissolveToPoint:NSMakePoint(35, 35) fraction:1.0];
 }
 - (void)mouseDown:(NSEvent *)theEvent
 {
-  if ([theEvent clickCount] > 1)
+  if ([theEvent clickCount] > 1) {
     [NSApp unhide:self];
+  } else {
+    /* Code copied from Switcher in GAP */
+    NSPoint lastLocation;
+    NSPoint location;
+    unsigned eventMask = NSLeftMouseDownMask | NSLeftMouseUpMask | NSPeriodicMask | NSOtherMouseUpMask | NSRightMouseUpMask;
+    NSDate *theDistantFuture = [NSDate distantFuture];
+    BOOL done = NO;
+
+    lastLocation = [theEvent locationInWindow];
+    [NSEvent startPeriodicEventsAfterDelay: 0.02 withPeriod: 0.02];
+
+    while (!done) {
+      theEvent = [NSApp nextEventMatchingMask: eventMask
+				    untilDate: theDistantFuture
+				       inMode: NSEventTrackingRunLoopMode
+				      dequeue: YES];
+      switch ([theEvent type]) {
+      case NSRightMouseUp:
+      case NSOtherMouseUp:
+      case NSLeftMouseUp:
+	done = YES;
+	break;
+      case NSPeriodic:
+	location = [_window mouseLocationOutsideOfEventStream];
+	if (NSEqualPoints(location, lastLocation) == NO) {
+	  NSPoint origin = [_window frame].origin;	  
+	  origin.x += (location.x - lastLocation.x);
+	  origin.y += (location.y - lastLocation.y);
+	  [_window setFrameOrigin: origin];
+	}
+	break;
+      default:
+	break;
+      }
+    }
+    [NSEvent stopPeriodicEvents];
+  }
 }
 @end
 
@@ -133,6 +206,8 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [soonStart changeDayBy:2];
   [soonEnd changeDayBy:5];
   while ((event = [enumerator nextObject])) {
+    if (![[event store] displayed])
+      continue;
     if ([event isScheduledForDay:today])
       [_today addChild:[DataTree dataTreeWithAttributes:[self attributesFrom:event and:today]]];
     if ([event isScheduledForDay:tomorrow])
@@ -147,7 +222,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [_tomorrow sortChildrenUsingFunction:compareDataTreeElements context:nil];
   [_soon sortChildrenUsingFunction:compareDataTreeElements context:nil];
   [_tasks removeChildren];
-  enumerator = [[_sm allTasks] objectEnumerator];
+  enumerator = [[_sm visibleTasks] objectEnumerator];
   while ((task = [enumerator nextObject])) {
     if ([task state] != TK_COMPLETED)
       [_tasks addChild:[DataTree dataTreeWithAttributes:[self attributesFromTask:task]]];
@@ -179,11 +254,8 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 {
   self = [super init];
   if (self) {
-    ConfigManager *config = [ConfigManager globalConfig];
-    [config registerDefaults:[self defaults]];
-    [config registerClient:self forKey:APPICON_DATE];
-    [config registerClient:self forKey:APPICON_TIME];
-    selectionManager = [SelectionManager globalManager];
+    [[ConfigManager globalConfig] registerDefaults:[self defaults]];
+    _selm = [SelectionManager globalManager];
     _sm = [StoreManager globalManager];
     _pc = [PreferencesController new];
     [self initSummary];
@@ -198,19 +270,6 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [[taskView tableColumnWithIdentifier:@"state"] setDataCell:cell];
   [[taskView tableColumnWithIdentifier:@"state"] setMaxWidth:128];
   [taskView setAutoresizesAllColumnsToFit:YES];
-  /* 
-   * FIXME : this shouldn't be needed but I can't make it
-   * work by editing the interface with Gorm...
-   * [[taskView superview] superview] is the ScrollView
-   * 
-   * Edit : I don't know if the bug comes from Gorm or
-   * NSTabView but here's an explanation : there is a NSView
-   * between NSTabView and each tab view (NSTabView -> NSView -> DayView,
-   * NSTabView -> NSView -> WeekView etc) and, except for the first tab,
-   * the intermediate view's autoresizingMask is 0
-   */
-  [[[[taskView superview] superview] superview] setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
-  [[weekView superview] setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
   [taskView setUsesAlternatingRowBackgroundColors:YES];
   [taskView setTarget:self];
   [taskView setDoubleAction:@selector(editAppointment:)];
@@ -218,6 +277,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [summary setTarget:self];
   [summary setDoubleAction:@selector(editAppointment:)];
   [window setFrameAutosaveName:@"mainWindow"];
+  [window makeKeyAndOrderFront:self];
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)not
@@ -242,7 +302,8 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SADataChangedInStoreManager object:nil];
   /* FIXME : this is overkill, we should only refresh the views for visual changes */
   [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SAStatusChangedForStore object:nil];
-  /* Set the selected day : this will update all views and titles (but not the summary */
+  /* Set the selected day : this will update all views and titles (but not the summary) */
+  [calendar setDateSource:self];
   [calendar setDate:[Date today]];
   /*
    * If stores are loaded before this is executed (it happens
@@ -252,23 +313,18 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
    * empty so this is needed here.
    */
   [self updateSummaryData];
+  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reminderWillRun:) name:SAEventReminderWillRun object:nil];
   /* This will init the alarms for all loaded elements needing one */
   [AlarmManager globalManager];
 }
 
-- (void)applicationWillTerminate:(NSNotification*)aNotification
+- (void)applicationWillTerminate:(NSNotification *)aNotification
 {
-  [[ConfigManager globalConfig] unregisterClient:self];
   [[NSNotificationCenter defaultCenter] removeObserver:self];
-  [_summaryRoot release];
-  [_pc release];
-  /* 
-   * FIXME : we shouldn't have to release the store
-   * manager as we don't retain it. It's a global
-   * instance that should synchronise itself when
-   * it's freed
-   */
-  [_sm release];
+  [_sm synchronise];
+  RELEASE(_summaryRoot);
+  RELEASE(_pc);
+  RELEASE(_appicon);
   RELEASE(_selectedDay);
 }
 
@@ -304,10 +360,16 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [_pc showPreferences];
 }
 
+NSComparisonResult compareEventTime(id a, id b, void *context)
+{
+  return [[a startDate] compareTime:[b startDate]];
+}
+
 - (int)_sensibleStartForDuration:(int)duration
 {
   int minute = [dayView firstHour] * 60;
-  NSEnumerator *enumerator = [[_sm scheduledAppointmentsForDay:_selectedDay] objectEnumerator];
+  NSEnumerator *enumerator = [[[[_sm visibleAppointmentsForDay:_selectedDay] allObjects] 
+				sortedArrayUsingFunction:compareEventTime context:nil] objectEnumerator];
   Event *apt;
 
   while ((apt = [enumerator nextObject])) {
@@ -371,7 +433,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 
 - (void)editAppointment:(id)sender
 {
-  id lastSelection = [selectionManager lastObject];
+  id lastSelection = [_selm lastObject];
 
   if (lastSelection) {
     if ([lastSelection isKindOfClass:[Event class]])
@@ -385,26 +447,26 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 
 - (void)delAppointment:(id)sender
 {
-  NSEnumerator *enumerator = [selectionManager enumerator];
+  NSEnumerator *enumerator = [_selm enumerator];
   Element *el;
 
   while ((el = [enumerator nextObject]))
     [[el store] remove:el];
-  [selectionManager clear];
+  [_selm clear];
 }
 
 - (void)exportAppointment:(id)sender;
 {
-  NSEnumerator *enumerator = [selectionManager enumerator];
+  NSEnumerator *enumerator = [_selm enumerator];
   NSSavePanel *panel = [NSSavePanel savePanel];
   NSString *str;
   iCalTree *tree;
   Element *el;
 
-  if ([selectionManager count] > 0) {
+  if ([_selm count] > 0) {
     [panel setRequiredFileType:@"ics"];
     [panel setTitle:_(@"Export as")];
-    if ([panel runModalForDirectory:nil file:[[selectionManager lastObject] summary]] == NSOKButton) {
+    if ([panel runModalForDirectory:nil file:[[_selm lastObject] summary]] == NSOKButton) {
       tree = [iCalTree new];
       while ((el = [enumerator nextObject]))
 	[tree add:el];
@@ -428,18 +490,18 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 
 - (void)copy:(id)sender
 {
-  [selectionManager copySelection];
+  [_selm copySelection];
 }
 
 - (void)cut:(id)sender
 {
-  [selectionManager cutSelection];
+  [_selm cutSelection];
 }
 
 - (void)paste:(id)sender
 {
-  if ([selectionManager copiedCount] > 0) {
-    NSEnumerator *enumerator = [[selectionManager paste] objectEnumerator];
+  if ([_selm copiedCount] > 0) {
+    NSEnumerator *enumerator = [[_selm paste] objectEnumerator];
     Date *date = [[calendar date] copy];
     Event *el;
     id <MemoryStore> store;
@@ -449,12 +511,17 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
     while ((el = [enumerator nextObject])) {
       /* FIXME : store property could be handled by Event:copy ? */
       store = [el store];
+
+      /* FIXME : this isn't enough : we have to find a writable store or error out */
+      if (![store writable])
+	store = [_sm defaultStore];
+	
       start = [[el startDate] minuteOfDay];
-      if ([selectionManager lastOperation] == SMCopy)
+      if ([_selm lastOperation] == SMCopy)
 	el = [el copy];
       [date setMinute:start];
       [el setStartDate:date];
-      if ([selectionManager lastOperation] == SMCopy) {
+      if ([_selm lastOperation] == SMCopy) {
 	[store add:el];
 	/*
 	 * FIXME : the new event is now in store's dictionary, we 
@@ -532,19 +599,29 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 - (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
 {
   SEL action = [menuItem action];
+  NSEnumerator *enumerator;
+  Element *el;
 
-  if (sel_eq(action, @selector(copy:)))
-    return [selectionManager count] > 0;
-  if (sel_eq(action, @selector(cut:)))
-    return [selectionManager count] > 0;
-  if (sel_eq(action, @selector(editAppointment:)))
-    return [selectionManager count] == 1;
-  if (sel_eq(action, @selector(delAppointment:)))
-    return [selectionManager count] > 0;
-  if (sel_eq(action, @selector(exportAppointment:)))
-    return [selectionManager count] > 0;
-  if (sel_eq(action, @selector(paste:)))
-    return [selectionManager copiedCount] > 0;
+  if (sel_isEqual(action, @selector(copy:)))
+    return [_selm count] > 0;
+  if (sel_isEqual(action, @selector(cut:))) {
+    if ([_selm count] == 0)
+      return NO;
+    enumerator = [[_selm selection] objectEnumerator];
+    while ((el = [enumerator nextObject])) {
+      if (![[el store] writable])
+	return NO;
+    }
+    return YES;
+  }
+  if (sel_isEqual(action, @selector(editAppointment:)))
+    return [_selm count] == 1;
+  if (sel_isEqual(action, @selector(delAppointment:)))
+    return [_selm count] > 0;
+  if (sel_isEqual(action, @selector(exportAppointment:)))
+    return [_selm count] > 0;
+  if (sel_isEqual(action, @selector(paste:)))
+    return [_selm copiedCount] > 0;
   return YES;
 }
 
@@ -554,6 +631,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
    * FIXME : if a selected event was deleted by another application, 
    * the selection will reference a non existing object
    */
+  [calendar reloadData];
   [dayView reloadData];
   [weekView reloadData];
   [taskView reloadData];
@@ -561,15 +639,44 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   [self updateSummaryData];
 }
 
+- (BOOL)showElement:(Element *)element onDay:(Date *)day
+{
+  NSString *tabIdentifier = [[tabs selectedTabViewItem] identifier];
+
+  NSAssert(element != nil, @"An object must be passed");
+  NSAssert([day isDate], @"This method uses a day, not a date");
+  if ([element isKindOfClass:[Event class]]) {
+    [calendar setDate:day];
+    if (![tabIdentifier isEqualToString:@"Day"] && ![tabIdentifier isEqualToString:@"Week"])
+      [tabs selectTabViewItemWithIdentifier:@"Day"];
+    [_selm select:element];
+    return YES;
+  }
+  if ([element isKindOfClass:[Task class]]) {
+    if (![tabIdentifier isEqualToString:@"Tasks"])
+      [tabs selectTabViewItemWithIdentifier:@"Tasks"];
+    [_selm select:element];
+    return YES;
+  }
+  return NO;
+}
+
+- (void)reminderWillRun:(NSNotification *)not
+{
+  Alarm *alarm = [not object];
+
+  [self showElement:[alarm element] onDay:[Date today]];
+}
+
 - (id)validRequestorForSendType:(NSString *)sendType returnType:(NSString *)returnType
 {
-  if ([selectionManager count] && (!sendType || [sendType isEqual:NSFilenamesPboardType] || [sendType isEqual:NSStringPboardType]))
+  if ([_selm count] && (!sendType || [sendType isEqual:NSFilenamesPboardType] || [sendType isEqual:NSStringPboardType]))
     return self;
   return nil;
 }
 - (BOOL)writeSelectionToPasteboard:(NSPasteboard *)pboard types:(NSArray *)types
 {
-  NSEnumerator *enumerator = [selectionManager enumerator];
+  NSEnumerator *enumerator = [_selm enumerator];
   Element *el;
   NSString *ical;
   NSString *filename;
@@ -577,7 +684,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   NSFileWrapper *fw;
   BOOL written;
 
-  if ([selectionManager count] == 0)
+  if ([_selm count] == 0)
     return NO;
   NSAssert([types count] == 1, @"It seems our assumption was wrong");
   tree = AUTORELEASE([iCalTree new]);
@@ -591,7 +698,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
       NSLog(@"Unable to encode into NSFileWrapper");
       return NO;
     }
-    filename = [NSString stringWithFormat:@"%@/%@.ics", NSTemporaryDirectory(), [[selectionManager lastObject] summary]];
+    filename = [NSString stringWithFormat:@"%@/%@.ics", NSTemporaryDirectory(), [[_selm lastObject] summary]];
     written = [fw writeToFile:filename atomically:YES updateFilenames:YES];
     [fw release];
     if (!written) {
@@ -607,11 +714,6 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   }
   return NO;
 }
-
-- (void)config:(ConfigManager *)config dataDidChangedForKey:(NSString *)key
-{
-  [_appicon setNeedsDisplay:YES];
-}
 @end
 
 @implementation AppController(NSOutlineViewDataSource)
@@ -643,29 +745,24 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 - (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item
 {
   id object = [item valueForKey:@"object"];
-  NSString *tabIdentifier = [[tabs selectedTabViewItem] identifier];
-  Date *date;
 
-  if (object && [object isKindOfClass:[Event class]]) {
-    date = [[item valueForKey:@"date"] copy];
-    [date setIsDate:YES];
-    [calendar setDate:date];
-    [date release];
-    if (![tabIdentifier isEqualToString:@"Day"] && ![tabIdentifier isEqualToString:@"Week"])
-      [tabs selectTabViewItemWithIdentifier:@"Day"];
-    [selectionManager set:object];
-    return YES;
-  }
-  if (object && [object isKindOfClass:[Task class]]) {
-    if (![tabIdentifier isEqualToString:@"Tasks"])
-      [tabs selectTabViewItemWithIdentifier:@"Tasks"];
-    [selectionManager set:object];
-    return YES;
-  }
+  if (object && [object isKindOfClass:[Event class]])
+    return [self showElement:object onDay:[Date dayWithDate:[item valueForKey:@"date"]]];
+  if (object && [object isKindOfClass:[Task class]])
+    return [self showElement:object onDay:[calendar date]];
   return NO;
 }
 @end
 
+@implementation AppController(CalendarViewDataSource)
+- (CVCellStatus)calendarView:(CalendarView *)view cellStatusForDate:(Date *)date
+{
+  if ([[_sm visibleAppointmentsForDay:date] count] > 0)
+    return CVHasDataCell;
+  return CVEmptyCell;
+}
+@end
+
 @implementation AppController(CalendarViewDelegate)
 - (void)calendarView:(CalendarView *)cs selectedDateChanged:(Date *)date
 {
@@ -730,11 +827,11 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 @implementation AppController(NSTableDataSource)
 - (int)numberOfRowsInTableView:(NSTableView *)aTableView
 {
-  return [[_sm allTasks] count];
+  return [[_sm visibleTasks] count];
 }
 - (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
 {
-  Task *task = [[_sm allTasks] objectAtIndex:rowIndex];
+  Task *task = [[_sm visibleTasks] objectAtIndex:rowIndex];
 
   if ([[aTableColumn identifier] isEqualToString:@"summary"])
     return [task summary];
@@ -742,7 +839,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 }
 - (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
 {
-  Task *task = [[_sm allTasks] objectAtIndex:rowIndex];
+  Task *task = [[_sm visibleTasks] objectAtIndex:rowIndex];
 
   if ([[task store] writable]) {
     [task setState:[anObject intValue]];
@@ -754,7 +851,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   Task *task;
 
   if ([[aTableColumn identifier] isEqualToString:@"state"]) {
-    task = [[_sm allTasks] objectAtIndex:rowIndex];
+    task = [[_sm visibleTasks] objectAtIndex:rowIndex];
     [aCell setEnabled:[[task store] writable]];
   }
 }
@@ -762,7 +859,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 {
   int index = [taskView selectedRow];
   if (index > -1)
-    [selectionManager set:[[_sm allTasks] objectAtIndex:index]];
+    [_selm select:[[_sm visibleTasks] objectAtIndex:index]];
 }
 @end
 
