@@ -21,6 +21,7 @@
   id timeText;
   Date *startDate;
   Event *_event;
+  NSArray *_modifiedAlarms;
 }
 
 + (AppointmentEditor *)editorForEvent:(Event *)event;
@@ -29,4 +30,5 @@
 - (void)selectFrequency:(id)sender;
 - (void)toggleUntil:(id)sender;
 - (void)toggleAllDay:(id)sender;
+- (void)editAlarms:(id)sender;
 @end
