@@ -4,14 +4,13 @@
 #import "defines.h"
 
 static NSImage *_repeatImage;
+static NSImage *_alarmImage;
 
 @implementation AppointmentView
 + (void)initialize
 {
-  if (!_repeatImage) {
-    _repeatImage = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForImageResource:@"repeat"]];
-    [_repeatImage setFlipped:YES];
-  }
+  _repeatImage = [NSImage imageNamed:@"repeat"];
+  _alarmImage = [NSImage imageNamed:@"small-bell"];
 }
 
 - (NSImage *)repeatImage
@@ -19,6 +18,11 @@ static NSImage *_repeatImage;
   return _repeatImage;
 }
 
+- (NSImage *)alarmImage
+{
+  return _alarmImage;
+}
+
 - (id)initWithFrame:(NSRect)frameRect appointment:(Event *)apt
 {
   self = [super initWithFrame:frameRect];
