@@ -12,6 +12,7 @@
 + (id)today;
 + (id)dateWithTimeInterval:(NSTimeInterval)seconds sinceDate:(Date *)refDate;
 + (id)dateWithCalendarDate:(NSCalendarDate *)cd withTime:(BOOL)time;
++ (id)dayWithDate:(Date *)date;
 - (NSCalendarDate *)calendarDate;
 - (NSComparisonResult)compare:(id)aDate;
 - (NSComparisonResult)compareTime:(id)aTime;
@@ -39,9 +40,12 @@
 - (void)setIsDate:(BOOL)date;
 - (NSTimeInterval)timeIntervalSince1970;
 - (NSTimeInterval)timeIntervalSinceDate:(Date *)anotherDate;
+- (NSTimeInterval)timeIntervalSinceNow;
+- (BOOL)belongsToDay:(Date *)day;
 @end
 
 @interface Date(iCalendar)
 - (id)initWithICalTime:(struct icaltimetype)time;
-- (struct icaltimetype)iCalTime;
+- (struct icaltimetype)UTCICalTime;
+- (struct icaltimetype)localICalTime;
 @end
