@@ -1,25 +1,30 @@
 #import <Foundation/Foundation.h>
 #import "Date.h"
 #import "Element.h"
-#import "SAAlarm.h"
+#import "Alarm.h"
 #import "NSString+SimpleAgenda.h"
 
 @implementation Element
 - (void)encodeWithCoder:(NSCoder *)coder
 {
   [coder encodeObject:_summary forKey:@"title"];
-  [coder encodeObject:[_text string] forKey:@"descriptionText"];
+  if (_text)
+    [coder encodeObject:[_text string] forKey:@"descriptionText"];
   [coder encodeObject:_uid forKey:@"uid"];
   [coder encodeInt:_classification forKey:@"classification"];
   if (_stamp)
     [coder encodeObject:_stamp forKey:@"dtstamp"];
   [coder encodeObject:_categories forKey:@"categories"];
+  [coder encodeObject:_alarms forKey:@"alarms"];
 }
 
 - (id)initWithCoder:(NSCoder *)coder
 {
   _summary = [[coder decodeObjectForKey:@"title"] retain];
-  _text = [[NSAttributedString alloc] initWithString:[coder decodeObjectForKey:@"descriptionText"]];
+  if ([coder containsValueForKey:@"descriptionText"])
+    _text = [[NSAttributedString alloc] initWithString:[coder decodeObjectForKey:@"descriptionText"]];
+  else
+    _text = nil;
   if ([coder containsValueForKey:@"uid"])
     _uid = [[coder decodeObjectForKey:@"uid"] retain];
   else
@@ -36,18 +41,19 @@
     _categories = [[coder decodeObjectForKey:@"categories"] retain];
   else
     _categories = [NSMutableArray new];
+  [self setAlarms:[coder decodeObjectForKey:@"alarms"]];
   return self;
 }
 
 - (id)init
 {
-  self = [super init];
-  if (self) {
-    _alarms = [NSMutableArray new];
-    _categories = [NSMutableArray new];
-    _classification = ICAL_CLASS_PUBLIC;
-    ASSIGNCOPY(_stamp, [Date now]);
-  }
+  if (!(self = [super init]))
+    return nil;
+  _alarms = [NSMutableArray new];
+  _categories = [NSMutableArray new];
+  _classification = ICAL_CLASS_PUBLIC;
+  _text = nil;
+  ASSIGNCOPY(_stamp, [Date now]);
   return self;
 }
 
@@ -151,17 +157,34 @@
   return [NSArray arrayWithArray:_alarms];
 }
 
-- (void)addAlarm:(SAAlarm *)alarm
+- (void)setAlarms:(NSArray *)alarms
+{
+  NSEnumerator *enumerator;
+  Alarm *alarm;
+
+  DESTROY(_alarms);
+  ASSIGNCOPY(_alarms, alarms);
+  enumerator = [_alarms objectEnumerator];
+  while ((alarm = [enumerator nextObject]))
+    [alarm setElement:self];
+}
+
+- (void)addAlarm:(Alarm *)alarm
 {
   [alarm setElement:self];
   [_alarms addObject:alarm];
 }
 
-- (void)removeAlarm:(SAAlarm *)alarm
+- (void)removeAlarm:(Alarm *)alarm
 {
   /* FIXME : do something */
 }
 
+- (Date *)nextActivationDate
+{
+  return nil;
+}
+
 - (NSArray *)categories
 {
   return [NSArray arrayWithArray:_categories];
@@ -188,6 +211,8 @@
 - (id)initWithICalComponent:(icalcomponent *)ic
 {
   icalproperty *prop;
+  icalcomponent *subc;
+  Alarm *alarm;
 
   self = [self init];
   if (self == nil)
@@ -216,7 +241,17 @@
   prop = icalcomponent_get_first_property(ic, ICAL_CATEGORIES_PROPERTY);
   if (prop)
     [self setCategories:[[NSString stringWithUTF8String:icalproperty_get_categories(prop)] componentsSeparatedByString:@","]];
+
+  subc = icalcomponent_get_first_component(ic, ICAL_VALARM_COMPONENT);
+  for (; subc != NULL; subc = icalcomponent_get_next_component(ic, ICAL_VALARM_COMPONENT)) {
+    alarm = [[Alarm alloc] initWithICalComponent:subc];
+    if (alarm) {
+      [self addAlarm:alarm];
+      RELEASE(alarm);
+    }
+  }
   return self;
+
  init_error:
   NSLog(@"Error creating Element from iCal component");
   [self release];
@@ -239,14 +274,17 @@
 
 - (void)deleteProperty:(icalproperty_kind)kind fromComponent:(icalcomponent *)ic
 {
-  icalproperty *prop;
-  prop = icalcomponent_get_first_property(ic, kind);
+  icalproperty *prop = icalcomponent_get_first_property(ic, kind);
   if (prop)
       icalcomponent_remove_property(ic, prop);
 }
 
 - (BOOL)updateICalComponent:(icalcomponent *)ic
 {
+  NSEnumerator *enumerator;
+  Alarm *alarm;
+  icalcomponent *subc;
+
   [self deleteProperty:ICAL_UID_PROPERTY fromComponent:ic];
   icalcomponent_add_property(ic, icalproperty_new_uid([[self UID] cString]));
   [self deleteProperty:ICAL_SUMMARY_PROPERTY fromComponent:ic];
@@ -256,11 +294,21 @@
   if ([self text])
     icalcomponent_add_property(ic, icalproperty_new_description([[[self text] string] UTF8String]));
   [self deleteProperty:ICAL_DTSTAMP_PROPERTY fromComponent:ic];
-  icalcomponent_add_property(ic, icalproperty_new_dtstamp([_stamp iCalTime]));  
+  icalcomponent_add_property(ic, icalproperty_new_dtstamp([_stamp UTCICalTime]));  
   [self deleteProperty:ICAL_CLASS_PROPERTY fromComponent:ic];
   icalcomponent_add_property(ic, icalproperty_new_class([self classification]));
   [self deleteProperty:ICAL_CATEGORIES_PROPERTY fromComponent:ic];
   icalcomponent_add_property(ic, icalproperty_new_categories([[[self categories] componentsJoinedByString:@","] UTF8String]));
+
+  subc = icalcomponent_get_first_component(ic, ICAL_VALARM_COMPONENT);
+  for (; subc != NULL; subc = icalcomponent_get_next_component(ic, ICAL_VALARM_COMPONENT))
+    icalcomponent_remove_component(ic, subc);
+  enumerator = [[self alarms] objectEnumerator];
+  while ((alarm = [enumerator nextObject])) {
+    subc = [alarm asICalComponent];
+    icalcomponent_add_component(ic, subc);
+    icalcomponent_free(subc);
+  }
   return YES;
 }
 
