@@ -2,50 +2,37 @@
 
 #import "Date.h"
 #import "Element.h"
-#import "AgendaStore.h"
-
-enum intervalType
-{
-  RI_NONE = 0, 
-  RI_DAILY, 
-  RI_WEEKLY, 
-  RI_MONTHLY, 
-  RI_YEARLY
-};
+#import "RecurrenceRule.h"
 
 @interface Event : Element
 {
   NSString *_location;
   BOOL _allDay;
-  Date *startDate;
-  Date *endDate;
-  int duration;
-  enum intervalType interval;
-  int frequency;
-  int scheduleLevel;
+  Date *_startDate;
+  int _duration;
+  RecurrenceRule *_rrule;
 }
 
 - (id)initWithStartDate:(Date *)start duration:(int)minutes title:(NSString *)aTitle;
 - (BOOL)isScheduledForDay:(Date *)day;
-- (BOOL)isScheduledBetweenDay:(Date *)start andDay:(Date *)end;
+- (NSRange)intersectionWithDay:(Date *)day;
 - (NSString *)details;
 - (BOOL)contains:(NSString *)text;
 
 - (NSString *)location;
 - (BOOL)allDay;
 - (int)duration;
-- (int)frequency;
 - (Date *)startDate;
-- (Date *)endDate;
-- (int)interval;
+- (RecurrenceRule *)rrule;
 
 - (void)setLocation:(NSString *)aLocation;
 - (void)setAllDay:(BOOL)allDay;
 - (void)setDuration:(int)duration;
-- (void)setFrequency:(int)frequency;
 - (void)setStartDate:(Date *)startDate;
-- (void)setEndDate:(Date *)endDate;
-- (void)setInterval:(int)interval;
+- (void)setRRule:(RecurrenceRule *)rrule;
+
+- (NSEnumerator *)dateEnumerator;
+- (NSEnumerator *)dateRangeEnumerator;
 @end
 
 @interface Event(iCalendar)
