@@ -7,7 +7,7 @@
 #import "iCalTree.h"
 #import "defines.h"
 
-@interface GroupDAVStore : MemoryStore <AgendaStore>
+@interface GroupDAVStore : MemoryStore <AgendaStore, ConfigListener>
 {
   NSURL *_url;
   WebDAVResource *_calendar;
@@ -110,7 +110,11 @@
 	set = (GSXPathNodeSet *)[xpc evaluateExpression:@"//response[propstat/prop/resourcetype/vtodo-collection]/href/text()"];
 	for (i = 0; i < [set count]; i++)
 	  [task addItemWithTitle:[[set nodeAtIndex:i] content]];
-	[xpc release];
+	RELEASE(xpc);
+	if ([calendar numberOfItems] > 0)
+	  [calendar selectItemAtIndex:1];
+	if ([task numberOfItems] > 0)
+	  [task selectItemAtIndex:1];
       }
     }
     [resource release];
@@ -145,8 +149,8 @@
 @implementation GroupDAVStore
 - (NSDictionary *)defaults
 {
-  return [NSDictionary dictionaryWithObjectsAndKeys:[[NSColor blueColor] description], ST_COLOR,
-		       [[NSColor darkGrayColor] description], ST_TEXT_COLOR,
+  return [NSDictionary dictionaryWithObjectsAndKeys:[[NSColor redColor] description], ST_COLOR,
+		       [[NSColor whiteColor] description], ST_TEXT_COLOR,
 		       [NSNumber numberWithBool:NO], ST_RW,
 		       [NSNumber numberWithBool:YES], ST_DISPLAY,
 		       [NSNumber numberWithBool:YES], ST_ENABLED,
@@ -162,6 +166,9 @@
     _hrefresource = [[NSMutableDictionary alloc] initWithCapacity:512];
     _modifiedhref = [NSMutableArray new];
     _loadedData = [[NSMutableSet alloc] initWithCapacity:512];
+    [_config registerClient:self forKey:ST_REFRESH];
+    [_config registerClient:self forKey:ST_REFRESH_INTERVAL];
+    [_config registerClient:self forKey:ST_ENABLED];
     [NSThread detachNewThreadSelector:@selector(initStoreAsync:) toTarget:self withObject:nil];
     [self initTimer];
   }
@@ -189,12 +196,12 @@
     if ([dialog task])
       taskURL = [NSURL URLWithString:[dialog task] possiblyRelativeToURL:baseURL];
     [dialog release];
-    cm = [[ConfigManager alloc] initForKey:name withParent:nil];
+    cm = [[ConfigManager alloc] initForKey:name];
     [cm setObject:[dialog url] forKey:ST_URL];
     if (calendarURL)
-      [cm setObject:[calendarURL description] forKey:ST_CALENDAR_URL];
+      [cm setObject:[calendarURL absoluteString] forKey:ST_CALENDAR_URL];
     if (taskURL)
-      [cm setObject:[taskURL description] forKey:ST_TASK_URL];
+      [cm setObject:[taskURL absoluteString] forKey:ST_TASK_URL];
     [cm setObject:[[self class] description] forKey:ST_CLASS];
     [cm setObject:[NSNumber numberWithBool:YES] forKey:ST_RW];
     return YES;
@@ -205,7 +212,7 @@
 
 + (NSString *)storeTypeName
 {
-  return @"GroupDAV store";
+  return @"GroupDAV";
 }
 
 - (void)dealloc
@@ -246,7 +253,7 @@
       /* Reloading all data is a way to handle href and uid modification done by the server */
       [self fetchData];
     else
-      NSLog(@"Error %d writing event to %@", [resource httpStatus], [url absoluteString]);
+      NSLog(@"Error %d writing event to %@", [resource httpStatus], [url anonymousAbsoluteString]);
   }
   [tree release];
   [resource release];
@@ -313,6 +320,14 @@
   [copy release];
   return YES;
 }
+
+- (void)config:(ConfigManager *)config dataDidChangedForKey:(NSString *)key
+{
+  if (config == _config && [key isEqualToString:ST_ENABLED] && [self enabled]) {
+    [self read];
+    [self initTimer];
+  }
+}
 @end
 
 
@@ -340,7 +355,7 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
       if (elementURL)
 	[result addObject:[elementURL absoluteString]];
     }
-    [xpc release];
+    RELEASE(xpc);
   }
   return result;
 }
@@ -390,6 +405,8 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
 }
 - (void)fetchData
 {
+  if (![self enabled])
+    return;
   [_loadedData removeAllObjects];
   if (_calendar)
     [self fetchList:[self itemsUnderRessource:_calendar]];
@@ -401,7 +418,7 @@ static NSString * const EXPRGETHREF = @"//response[propstat/prop/getetag]/href/t
     else
       [self performSelectorOnMainThread:@selector(fillWithElements:) withObject:_loadedData waitUntilDone:YES];
   }      
-  NSLog(@"GroupDAVStore from %@ : loaded %d appointment(s)", [_url absoluteString], [[self events] count]);
-  NSLog(@"GroupDAVStore from %@ : loaded %d tasks(s)", [_url absoluteString], [[self tasks] count]);
+  NSLog(@"GroupDAVStore from %@ : loaded %d appointment(s)", [_url anonymousAbsoluteString], [[self events] count]);
+  NSLog(@"GroupDAVStore from %@ : loaded %d tasks(s)", [_url anonymousAbsoluteString], [[self tasks] count]);
 }
 @end
