@@ -9,7 +9,8 @@ extern NSString * const SADataChangedInStoreManager;
 @interface StoreManager : NSObject
 {
   NSMutableDictionary *_stores;
-  NSMutableDictionary *_cache;
+  NSMutableDictionary *_dayEventsCache;
+  NSMutableArray *_eventCache;
   id _defaultStore;
 }
 
@@ -23,11 +24,12 @@ extern NSString * const SADataChangedInStoreManager;
 - (void)setDefaultStore:(NSString *)name;
 - (id <AgendaStore>)defaultStore;
 - (NSEnumerator *)storeEnumerator;
-/* FIXME : add enabledStoreEnumerator ? */
 - (void)synchronise;
 - (void)refresh;
 - (id <AgendaStore>)storeContainingElement:(Element *)elt;
 - (NSArray *)allEvents;
 - (NSArray *)allTasks;
 - (NSSet *)scheduledAppointmentsForDay:(Date *)date;
+- (NSSet *)visibleAppointmentsForDay:(Date *)date;
+- (NSArray *)visibleTasks;
 @end
