@@ -5,24 +5,25 @@
 
 enum taskState
 {
-  TK_NONE = 0, 
-  TK_INPROCESS, 
-  TK_COMPLETED, 
-  TK_CANCELED 
+  TK_NONE = 0,
+  TK_INPROCESS,
+  TK_COMPLETED,
+  TK_CANCELED,
+  TK_NEEDSACTION
 };
 
 @interface Task : Element
 {
   enum taskState _state;
-  Date *_completionDate;
+  Date *_dueDate;
 }
 
 + (NSArray *)stateNamesArray;
 - (enum taskState)state;
 - (NSString *)stateAsString;
 - (void)setState:(enum taskState)state;
-- (Date *)completionDate;
-- (void)setCompletionDate:(Date *)cd;
+- (Date *)dueDate;
+- (void)setDueDate:(Date *)cd;
 @end
 
 @interface Task(iCalendar)
