@@ -12,10 +12,17 @@
   id store;
   id state;
   id ok;
+  id toggleDueDate;
+  id dueDate;
+  id dueTime;
+  id alarms;
   Task *_task;
+  NSArray *_modifiedAlarms;
 }
 
 + (TaskEditor *)editorForTask:(Task *)task;
 - (void)validate:(id)sender;
 - (void)cancel:(id)sender;
+- (void)editAlarms:(id)sender;
+- (void)toggleDueDate:(id)sender;
 @end
