@@ -4,6 +4,9 @@
 #import "TaskEditor.h"
 #import "StoreManager.h"
 #import "Task.h"
+#import "AlarmEditor.h"
+#import "HourFormatter.h"
+#import "Date.h"
 
 static NSMutableDictionary *editors;
 
@@ -16,10 +19,16 @@ static NSMutableDictionary *editors;
 
 - (id)init
 {
-  self = [super init];
-  if (self) {
-    if (![NSBundle loadNibNamed:@"Task" owner:self])
-      return nil;
+  HourFormatter *formatter;
+  NSDateFormatter *dateFormatter;
+
+  if (![NSBundle loadNibNamed:@"Task" owner:self])
+    return nil;
+  if ((self = [super init])) {
+    formatter = AUTORELEASE([[HourFormatter alloc] init]);
+    dateFormatter = AUTORELEASE([[NSDateFormatter alloc] initWithDateFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSShortDateFormatString] allowNaturalLanguage:NO]);
+    [dueTime setFormatter:formatter];
+    [dueDate setFormatter:dateFormatter];
   }
   return self;
 }
@@ -34,6 +43,7 @@ static NSMutableDictionary *editors;
   self = [self init];
   if (self) {
     ASSIGN(_task, task);
+    ASSIGNCOPY(_modifiedAlarms, [task alarms]);
     [summary setStringValue:[task summary]];
 
     [[description textStorage] deleteCharactersInRange:NSMakeRange(0, [[description textStorage] length])];
@@ -56,6 +66,13 @@ static NSMutableDictionary *editors;
     [state addItemsWithTitles:[Task stateNamesArray]];
     [state selectItemWithTitle:[task stateAsString]];
     [ok setEnabled:[self canBeModified]];
+    if ([task dueDate]) {
+      [dueDate setObjectValue:[[task dueDate] calendarDate]];
+      [dueTime setIntValue:[[task dueDate] hourOfDay] * 3600 + [[task dueDate] minuteOfHour] * 60];
+      [toggleDueDate setState:YES];
+    }
+    [dueDate setEnabled:[toggleDueDate state]];
+    [dueTime setEnabled:[toggleDueDate state]];
     [window makeKeyAndOrderFront:self];
   }
   return self;
@@ -63,8 +80,9 @@ static NSMutableDictionary *editors;
 
 - (void)dealloc 
 { 
-  RELEASE(_task); 
-  [super dealloc]; 
+  RELEASE(_task);
+  RELEASE(_modifiedAlarms);
+  [super dealloc];
 } 
 
 + (void)initialize
@@ -90,10 +108,19 @@ static NSMutableDictionary *editors;
   StoreManager *sm = [StoreManager globalManager];
   id <MemoryStore> originalStore = [_task store];
   id <MemoryStore> aStore;
+  Date *date;
 
   [_task setSummary:[summary stringValue]];
   [_task setText:[[description textStorage] copy]];
   [_task setState:[state indexOfSelectedItem]];
+  [_task setAlarms:_modifiedAlarms];
+  if ([toggleDueDate state]) {
+    date = [Date dateWithCalendarDate:[dueDate objectValue] withTime:NO];
+    date = [Date dateWithTimeInterval:[dueTime intValue] sinceDate:date];
+    [_task setDueDate:date];
+  } else {
+    [_task setDueDate:nil];
+  }
   aStore = [sm storeForName:[store titleOfSelectedItem]];
   if (!originalStore)
     [aStore add:_task];
@@ -103,14 +130,41 @@ static NSMutableDictionary *editors;
     [originalStore remove:_task];
     [aStore add:_task];
   }
-  [editors removeObjectForKey:[_task UID]];
   [window close];
+  [editors removeObjectForKey:[_task UID]];
 }
 
 - (void)cancel:(id)sender
 {
-  [editors removeObjectForKey:[_task UID]]; 
   [window close]; 
+  [editors removeObjectForKey:[_task UID]]; 
+}
+
+- (void)editAlarms:(id)sender
+{
+  NSArray *alarmArray;
+
+  alarmArray = [AlarmEditor editAlarms:_modifiedAlarms];
+  if (alarmArray)
+    ASSIGN(_modifiedAlarms, alarmArray);
+  [window makeKeyAndOrderFront:self];
+}
+
+- (void)toggleDueDate:(id)sender
+{
+  Date *date;
+
+  [dueDate setEnabled:[toggleDueDate state]];
+  [dueTime setEnabled:[toggleDueDate state]];
+  if ([toggleDueDate state]) {
+    date = [Date now];
+    [date changeDayBy:7];
+    [dueDate setObjectValue:[date calendarDate]];
+    [dueTime setIntValue:[date hourOfDay] * 3600 + [date minuteOfHour] * 60];
+  } else {
+    [dueDate setObjectValue:nil];
+    [dueTime setObjectValue:nil];
+  }
 }
 
 - (BOOL)textView:(NSTextView *)aTextView doCommandBySelector:(SEL)aSelector
