@@ -16,12 +16,10 @@
   NSString *_user;
   NSString *_password;
   NSData *_data;
-  BOOL _debug;
 }
 
 - (id)initWithURL:(NSURL *)url;
 - (id)initWithURL:(NSURL *)anUrl authFromURL:(NSURL *)parent;
-- (void)setDebug:(BOOL)debug;
 - (BOOL)readable;
 /* WARNING Destructive */
 - (BOOL)writableWithData:(NSData *)data;
@@ -38,7 +36,7 @@
 @interface NSURL(SimpleAgenda)
 + (BOOL)stringIsValidURL:(NSString *)string;
 + (NSURL *)URLWithString:(NSString *)string possiblyRelativeToURL:(NSURL *)base;
-- (NSURL *)redirection;
+- (NSString *)anonymousAbsoluteString;
 @end
 
 @interface GSXMLDocument(SimpleAgenda)
