@@ -3,15 +3,6 @@
 #import "WebDAVResource.h"
 
 @implementation WebDAVResource
-- (void)debugLog:(NSString *)format,...
-{
-  va_list ap;
-  if (_debug) {
-    va_start(ap, format);
-    NSLogv(format, ap);
-    va_end(ap);
-  }
-}
 - (void)dealloc
 {
   DESTROY(_user);
@@ -20,28 +11,31 @@
   DESTROY(_url);
   DESTROY(_lastModified);
   DESTROY(_data);
+  DESTROY(_reason);
+  DESTROY(_etag);
   [super dealloc];
 }
 
-- (void)fixURLScheme
+- (void)setURL:(NSURL *)anURL
 {
-  if ([[_url scheme] hasPrefix:@"webcal"])
-    ASSIGN(_url, [NSURL URLWithString:[[_url absoluteString] stringByReplacingString:@"webcal" withString:@"http"]]);
+  if ([[anURL scheme] hasPrefix:@"webcal"])
+    ASSIGN(_url, [NSURL URLWithString:[[anURL absoluteString] stringByReplacingString:@"webcal" withString:@"http"]]);
+  else
+    ASSIGN(_url, anURL);
+  _handleClass = [NSURLHandle URLHandleClassForURL:_url];
+  if ([_url user])
+    ASSIGN(_user, [_url user]);
+  if ([_url password])
+    ASSIGN(_password, [_url password]);
 }
 
 - (id)initWithURL:(NSURL *)anUrl
 {
   self = [super init];
   if (self) {
-    /* FIXME : this causes a bogus GET for every resource creation */
-    _url = [anUrl redirection];
-    [self fixURLScheme];
-    _handleClass = [NSURLHandle URLHandleClassForURL:_url];
     _lock = [NSLock new];
-    _user = nil;
-    _password = nil;
     _data = nil;
-    _debug = NO;
+    [self setURL:anUrl];
   }
   return self;
 }
@@ -56,11 +50,6 @@
   return self;
 }
 
-- (void)setDebug:(BOOL)debug
-{
-  _debug = debug;
-}
-
 /* FIXME : ugly hack to work around NSURLHandle shortcomings */
 - (NSString *)basicAuth
 {
@@ -85,6 +74,7 @@
   NSURLHandle *handle;
 
   [_lock lock];
+ restart:
   handle = [[_handleClass alloc] initWithURL:_url cached:NO];
   [handle writeProperty:method forKey:GSHTTPPropertyMethodKey];
   if (attributes) {
@@ -98,7 +88,7 @@
     [handle writeProperty:[NSString stringWithFormat:@"([%@])", _etag] forKey:@"If"];
   if (body)
     [handle writeData:body];
-  [self debugLog:@"%@ %@ (%@)", [_url absoluteString], method, [attributes description]];
+  NSDebugLog(@"%@ %@ (%@)", [_url anonymousAbsoluteString], method, [attributes description]);
   DESTROY(_data);
   data = [handle resourceData];
   /* FIXME : this is more than ugly */
@@ -106,10 +96,15 @@
     _httpStatus = data ? 200 : 199;
   else
     _httpStatus = [[handle propertyForKeyIfAvailable:NSHTTPPropertyStatusCodeKey] intValue];
-  if (data)
-    [self debugLog:@"%@ =>\n%@", method, AUTORELEASE([[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding])];
-  else
-    [self debugLog:@"%@ status %d", method, _httpStatus];
+
+  /* FIXME : why do we have to check for httpStatus == 0 */
+  if ((_httpStatus == 0 ||_httpStatus == 301 || _httpStatus == 302) && [handle propertyForKey:@"Location"] != nil) {
+    NSDebugLog(@"Redirection to %@", [handle propertyForKey:@"Location"]);
+    [self setURL:[NSURL URLWithString:[handle propertyForKey:@"Location"]]];
+    goto restart;
+  }
+
+  NSDebugLog(@"%@ status %d", method, _httpStatus);
   property = [handle propertyForKeyIfAvailable:NSHTTPPropertyStatusReasonKey];
   if (property)
     ASSIGN(_reason, property);
@@ -233,6 +228,7 @@ static NSString * const GETLASTMODIFIED = @"string(/multistatus/response/propsta
       result = (GSXPathString *)[xpc evaluateExpression:GETLASTMODIFIED];
       if (result)
 	ASSIGN(_lastModified, [result stringValue]);
+      RELEASE(xpc);
     }
   }
 }
@@ -246,8 +242,9 @@ static NSString * const GETLASTMODIFIED = @"string(/multistatus/response/propsta
 
   NS_DURING
     {
-      url = [NSURL URLWithString:string];
-      valid = url ? YES : NO;
+      /* We want an absolute URL */
+      if ((url = [NSURL URLWithString:string]) && [url scheme])
+	  valid = YES;
     }
   NS_HANDLER
     {
@@ -262,19 +259,17 @@ static NSString * const GETLASTMODIFIED = @"string(/multistatus/response/propsta
   if ([NSURL stringIsValidURL:string])
     url = [NSURL URLWithString:string];
   else
-    url = [NSURL URLWithString:[[base absoluteString] stringByReplacingString:[base path] withString:string]];
+    url = [NSURL URLWithString:string relativeToURL:base];
   return url;
 }
-- (NSURL *)redirection
+- (NSString *)anonymousAbsoluteString
 {
-  NSString *location;
+  NSString *as = [self absoluteString];
 
-  location = [self propertyForKey:@"Location"];
-  if (location) {
-    NSLog(@"Redirected to %@", location);
-    return [[NSURL URLWithString:location] redirection];
-  }
-  return [self copy];
+  if (![self user] && ![self password])
+    return as;
+  return [as stringByReplacingOccurrencesOfString:[NSString stringWithFormat:@"%@:%@@", [self user], [self password]]
+				       withString:@""];
 }
 @end
 
