@@ -8,10 +8,8 @@
 {
   self = [super init];
   if (self) {
-    root = icalcomponent_vanew(ICAL_VCALENDAR_COMPONENT,
-			       icalproperty_new_version("1.0"),
-			       icalproperty_new_prodid("-//Octets//NONSGML SimpleAgenda Calendar//EN"),
-			       0);
+    root = icalcomponent_vanew(ICAL_VCALENDAR_COMPONENT, icalproperty_new_version("1.0"),
+			       icalproperty_new_prodid("-//Octets//NONSGML SimpleAgenda Calendar//EN"), 0);
     if (!root) {
       [self release];
       return nil;
@@ -22,19 +20,17 @@
 
 - (void)dealloc
 {
+  icalcomponent_free(root);
   [super dealloc];
-  if (root)
-    icalcomponent_free(root);
 }
 
 - (BOOL)parseString:(NSString *)string;
 {
   icalcomponent *icomp;
 
-  icomp = icalparser_parse_string([string cStringUsingEncoding:NSUTF8StringEncoding]);
+  icomp = icalparser_parse_string([string UTF8String]);
   if (icomp) {
-    if (root)
-      icalcomponent_free(root);
+    icalcomponent_free(root);
     root = icomp;
     return YES;
   }
@@ -43,10 +39,7 @@
 
 - (BOOL)parseData:(NSData *)data
 {
-  NSString *text;
-
-  text = AUTORELEASE([[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
-  return [self parseString:text];
+  return [self parseString:AUTORELEASE([[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding])];
 }
 
 - (NSString *)iCalTreeAsString
@@ -130,8 +123,10 @@
 - (BOOL)update:(Element *)elt
 {
   icalcomponent *ic = [self componentForEvent:elt];
-  if (!ic)
+  if (!ic) {
+    NSLog(@"No iCal component found for %@", [elt description]);
     return NO;
+  }
   return [elt updateICalComponent:ic];
 }
 @end
